@@ -24,12 +24,13 @@ from matplotlib.backend_bases import (
     TimerBase, ToolContainerBase, cursors,
     CloseEvent, KeyEvent, LocationEvent, MouseEvent, ResizeEvent)
 
-from matplotlib import _api, cbook, backend_tools
+from matplotlib import _api, cbook, backend_tools, _c_internal_utils
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.path import Path
 from matplotlib.transforms import Affine2D
 
 import wx
+import wx.svg  # noqa: F401
 
 _log = logging.getLogger(__name__)
 
@@ -45,6 +46,8 @@ def _create_wxapp():
     wxapp = wx.App(False)
     wxapp.SetExitOnFrameDelete(True)
     cbook._setup_new_guiapp()
+    # Set per-process DPI awareness. This is a NoOp except in MSW
+    _c_internal_utils.Win32_SetProcessDpiAwareness_max()
     return wxapp
 
 
@@ -471,12 +474,12 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         """
 
         FigureCanvasBase.__init__(self, figure)
-        w, h = map(math.ceil, self.figure.bbox.size)
+        size = wx.Size(*map(math.ceil, self.figure.bbox.size))
+        if wx.Platform != '__WXMSW__':
+            size = parent.FromDIP(size)
         # Set preferred window size hint - helps the sizer, if one is connected
-        wx.Panel.__init__(self, parent, id, size=wx.Size(w, h))
-        # Create the drawing bitmap
-        self.bitmap = wx.Bitmap(w, h)
-        _log.debug("%s - __init__() - bitmap w:%d h:%d", type(self), w, h)
+        wx.Panel.__init__(self, parent, id, size=size)
+        self.bitmap = None
         self._isDrawn = False
         self._rubberband_rect = None
         self._rubberband_pen_black = wx.Pen('BLACK', 1, wx.PENSTYLE_SHORT_DASH)
@@ -512,6 +515,12 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         self.SetBackgroundStyle(wx.BG_STYLE_PAINT)  # Reduce flicker.
         self.SetBackgroundColour(wx.WHITE)
 
+        if wx.Platform == '__WXMAC__':
+            # Initial scaling. Other platforms handle this automatically
+            dpiScale = self.GetDPIScaleFactor()
+            self.SetInitialSize(self.GetSize()*(1/dpiScale))
+            self._set_device_pixel_ratio(dpiScale)
+
     def Copy_to_Clipboard(self, event=None):
         """Copy bitmap of canvas to system clipboard."""
         bmp_obj = wx.BitmapDataObject()
@@ -524,6 +533,12 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
                 wx.TheClipboard.Flush()
                 wx.TheClipboard.Close()
 
+    def _update_device_pixel_ratio(self, *args, **kwargs):
+        # We need to be careful in cases with mixed resolution displays if
+        # device_pixel_ratio changes.
+        if self._set_device_pixel_ratio(self.GetDPIScaleFactor()):
+            self.draw()
+
     def draw_idle(self):
         # docstring inherited
         _log.debug("%s - draw_idle()", type(self))
@@ -631,7 +646,7 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         In this application we attempt to resize to fit the window, so it
         is better to take the performance hit and redraw the whole window.
         """
-
+        self._update_device_pixel_ratio()
         _log.debug("%s - _on_size()", type(self))
         sz = self.GetParent().GetSizer()
         if sz:
@@ -655,9 +670,10 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
             return  # Empty figure
 
         # Create a new, correctly sized bitmap
-        self.bitmap = wx.Bitmap(self._width, self._height)
-
         dpival = self.figure.dpi
+        if not wx.Platform == '__WXMSW__':
+            scale = self.GetDPIScaleFactor()
+            dpival /= scale
         winch = self._width / dpival
         hinch = self._height / dpival
         self.figure.set_size_inches(winch, hinch, forward=False)
@@ -669,6 +685,22 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         ResizeEvent("resize_event", self)._process()
         self.draw_idle()
 
+    @staticmethod
+    def _mpl_buttons():
+        state = wx.GetMouseState()
+        # NOTE: Alternatively, we could use event.LeftIsDown() / etc. but this
+        # fails to report multiclick drags on macOS (other OSes have not been
+        # verified).
+        mod_table = [
+            (MouseButton.LEFT, state.LeftIsDown()),
+            (MouseButton.RIGHT, state.RightIsDown()),
+            (MouseButton.MIDDLE, state.MiddleIsDown()),
+            (MouseButton.BACK, state.Aux1IsDown()),
+            (MouseButton.FORWARD, state.Aux2IsDown()),
+        ]
+        # State *after* press/release.
+        return {button for button, flag in mod_table if flag}
+
     @staticmethod
     def _mpl_modifiers(event=None, *, exclude=None):
         mod_table = [
@@ -712,7 +744,11 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         else:
             x, y = pos.X, pos.Y
         # flip y so y=0 is bottom of canvas
-        return x, self.figure.bbox.height - y
+        if not wx.Platform == '__WXMSW__':
+            scale = self.GetDPIScaleFactor()
+            return x*scale, self.figure.bbox.height - y*scale
+        else:
+            return x, self.figure.bbox.height - y
 
     def _on_key_down(self, event):
         """Capture key press."""
@@ -774,9 +810,8 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
             MouseEvent("button_press_event", self, x, y, button,
                        modifiers=modifiers, guiEvent=event)._process()
         elif event.ButtonDClick():
-            MouseEvent("button_press_event", self, x, y, button,
-                       dblclick=True, modifiers=modifiers,
-                       guiEvent=event)._process()
+            MouseEvent("button_press_event", self, x, y, button, dblclick=True,
+                       modifiers=modifiers, guiEvent=event)._process()
         elif event.ButtonUp():
             MouseEvent("button_release_event", self, x, y, button,
                        modifiers=modifiers, guiEvent=event)._process()
@@ -806,6 +841,7 @@ class _FigureCanvasWxBase(FigureCanvasBase, wx.Panel):
         event.Skip()
         MouseEvent("motion_notify_event", self,
                    *self._mpl_coords(event),
+                   buttons=self._mpl_buttons(),
                    modifiers=self._mpl_modifiers(event),
                    guiEvent=event)._process()
 
@@ -898,8 +934,8 @@ class FigureFrameWx(wx.Frame):
         # On Windows, canvas sizing must occur after toolbar addition;
         # otherwise the toolbar further resizes the canvas.
         w, h = map(math.ceil, fig.bbox.size)
-        self.canvas.SetInitialSize(wx.Size(w, h))
-        self.canvas.SetMinSize((2, 2))
+        self.canvas.SetInitialSize(self.FromDIP(wx.Size(w, h)))
+        self.canvas.SetMinSize(self.FromDIP(wx.Size(2, 2)))
         self.canvas.SetFocus()
 
         self.Fit()
@@ -1017,9 +1053,9 @@ def _set_frame_icon(frame):
 class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
     def __init__(self, canvas, coordinates=True, *, style=wx.TB_BOTTOM):
         wx.ToolBar.__init__(self, canvas.GetParent(), -1, style=style)
+        if wx.Platform == '__WXMAC__':
+            self.SetToolBitmapSize(self.GetToolBitmapSize()*self.GetDPIScaleFactor())
 
-        if 'wxMac' in wx.PlatformInfo:
-            self.SetToolBitmapSize((24, 24))
         self.wx_ids = {}
         for text, tooltip_text, image_file, callback in self.toolitems:
             if text is None:
@@ -1028,7 +1064,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
             self.wx_ids[text] = (
                 self.AddTool(
                     -1,
-                    bitmap=self._icon(f"{image_file}.png"),
+                    bitmap=self._icon(f"{image_file}.svg"),
                     bmpDisabled=wx.NullBitmap,
                     label=text, shortHelp=tooltip_text,
                     kind=(wx.ITEM_CHECK if text in ["Pan", "Zoom"]
@@ -1054,9 +1090,6 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
         *name*, including the extension and relative to Matplotlib's "images"
         data directory.
         """
-        pilimg = PIL.Image.open(cbook._get_data_path("images", name))
-        # ensure RGBA as wx BitMap expects RGBA format
-        image = np.array(pilimg.convert("RGBA"))
         try:
             dark = wx.SystemSettings.GetAppearance().IsDark()
         except AttributeError:  # wxpython < 4.1
@@ -1067,12 +1100,24 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
             bg_lum = (.299 * bg.red + .587 * bg.green + .114 * bg.blue) / 255
             fg_lum = (.299 * fg.red + .587 * fg.green + .114 * fg.blue) / 255
             dark = fg_lum - bg_lum > .2
-        if dark:
-            fg = wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOWTEXT)
-            black_mask = (image[..., :3] == 0).all(axis=-1)
-            image[black_mask, :3] = (fg.Red(), fg.Green(), fg.Blue())
-        return wx.Bitmap.FromBufferRGBA(
-            image.shape[1], image.shape[0], image.tobytes())
+
+        path = cbook._get_data_path('images', name)
+        if path.suffix == '.svg':
+            svg = path.read_bytes()
+            if dark:
+                svg = svg.replace(b'fill:black;', b'fill:white;')
+            toolbarIconSize = wx.ArtProvider().GetDIPSizeHint(wx.ART_TOOLBAR)
+            return wx.BitmapBundle.FromSVG(svg, toolbarIconSize)
+        else:
+            pilimg = PIL.Image.open(path)
+            # ensure RGBA as wx BitMap expects RGBA format
+            image = np.array(pilimg.convert("RGBA"))
+            if dark:
+                fg = wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOWTEXT)
+                black_mask = (image[..., :3] == 0).all(axis=-1)
+                image[black_mask, :3] = (fg.Red(), fg.Green(), fg.Blue())
+            return wx.Bitmap.FromBufferRGBA(
+                image.shape[1], image.shape[0], image.tobytes())
 
     def _update_buttons_checked(self):
         if "Pan" in self.wx_ids:
@@ -1114,6 +1159,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
                 mpl.rcParams["savefig.directory"] = str(path.parent)
             try:
                 self.canvas.figure.savefig(path, format=fmt)
+                return path
             except Exception as e:
                 dialog = wx.MessageDialog(
                     parent=self.canvas.GetParent(), message=str(e),
@@ -1123,7 +1169,9 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
 
     def draw_rubberband(self, event, x0, y0, x1, y1):
         height = self.canvas.figure.bbox.height
-        self.canvas._rubberband_rect = (x0, height - y0, x1, height - y1)
+        sf = 1 if wx.Platform == '__WXMSW__' else self.canvas.GetDPIScaleFactor()
+        self.canvas._rubberband_rect = (x0/sf, (height - y0)/sf,
+                                        x1/sf, (height - y1)/sf)
         self.canvas.Refresh()
 
     def remove_rubberband(self):
@@ -1146,6 +1194,8 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
 # tools for matplotlib.backend_managers.ToolManager:
 
 class ToolbarWx(ToolContainerBase, wx.ToolBar):
+    _icon_extension = '.svg'
+
     def __init__(self, toolmanager, parent=None, style=wx.TB_BOTTOM):
         if parent is None:
             parent = toolmanager.canvas.GetParent()
@@ -1164,8 +1214,8 @@ class ToolbarWx(ToolContainerBase, wx.ToolBar):
         ``ToolBar.GetToolPos`` is not useful because wx assigns the same Id to
         all Separators and StretchableSpaces.
         """
-        pos, = [pos for pos in range(self.ToolsCount)
-                if self.GetToolByPos(pos) == tool]
+        pos, = (pos for pos in range(self.ToolsCount)
+                if self.GetToolByPos(pos) == tool)
         return pos
 
     def add_toolitem(self, name, group, position, image_file, description,
@@ -1224,9 +1274,8 @@ class ToolbarWx(ToolContainerBase, wx.ToolBar):
         self.Refresh()
 
     def remove_toolitem(self, name):
-        for tool, handler in self._toolitems[name]:
+        for tool, handler in self._toolitems.pop(name, []):
             self.DeleteTool(tool.Id)
-        del self._toolitems[name]
 
     def set_message(self, s):
         self._label_text.SetLabel(s)
