@@ -2,13 +2,19 @@ import io
 
 import numpy as np
 from numpy.testing import assert_array_almost_equal
+from PIL import features, Image, TiffTags
 import pytest
 
+
 from matplotlib import (
-    collections, path, pyplot as plt, transforms as mtransforms, rcParams)
-from matplotlib.image import imread
+    collections, patheffects, pyplot as plt, transforms as mtransforms,
+    rcParams, rc_context)
+from matplotlib.backends.backend_agg import RendererAgg
 from matplotlib.figure import Figure
+from matplotlib.image import imread
+from matplotlib.path import Path
 from matplotlib.testing.decorators import image_comparison
+from matplotlib.transforms import IdentityTransform
 
 
 def test_repeated_save_with_alpha():
@@ -50,7 +56,7 @@ def test_large_single_path_collection():
     # applied.
     f, ax = plt.subplots()
     collection = collections.PathCollection(
-        [path.Path([[-10, 5], [10, 5], [10, -5], [-10, -5], [-10, 5]])])
+        [Path([[-10, 5], [10, 5], [10, -5], [-10, -5], [-10, 5]])])
     ax.add_artist(collection)
     ax.set_xlim(10**-3, 1)
     plt.savefig(buff)
@@ -70,10 +76,10 @@ def test_marker_with_nan():
 
 def test_long_path():
     buff = io.BytesIO()
-
-    fig, ax = plt.subplots()
-    np.random.seed(0)
-    points = np.random.rand(70000)
+    fig = Figure()
+    ax = fig.subplots()
+    points = np.ones(100_000)
+    points[::2] *= -1
     ax.plot(points)
     fig.savefig(buff, format='png')
 
@@ -81,7 +87,7 @@ def test_long_path():
 @image_comparison(['agg_filter.png'], remove_text=True)
 def test_agg_filter():
     def smooth1d(x, window_len):
-        # copied from http://www.scipy.org/Cookbook/SignalSmooth
+        # copied from https://scipy-cookbook.readthedocs.io/
         s = np.r_[
             2*x[0] - x[window_len:1:-1], x, 2*x[-1] - x[-1:-window_len:-1]]
         w = np.hanning(window_len)
@@ -99,7 +105,7 @@ def test_agg_filter():
         def get_pad(self, dpi):
             return 0
 
-        def process_image(padded_src, dpi):
+        def process_image(self, padded_src, dpi):
             raise NotImplementedError("Should be overridden by subclasses")
 
         def __call__(self, im, dpi):
@@ -159,30 +165,29 @@ def test_agg_filter():
     fig, ax = plt.subplots()
 
     # draw lines
-    l1, = ax.plot([0.1, 0.5, 0.9], [0.1, 0.9, 0.5], "bo-",
-                  mec="b", mfc="w", lw=5, mew=3, ms=10, label="Line 1")
-    l2, = ax.plot([0.1, 0.5, 0.9], [0.5, 0.2, 0.7], "ro-",
-                  mec="r", mfc="w", lw=5, mew=3, ms=10, label="Line 1")
+    line1, = ax.plot([0.1, 0.5, 0.9], [0.1, 0.9, 0.5], "bo-",
+                     mec="b", mfc="w", lw=5, mew=3, ms=10, label="Line 1")
+    line2, = ax.plot([0.1, 0.5, 0.9], [0.5, 0.2, 0.7], "ro-",
+                     mec="r", mfc="w", lw=5, mew=3, ms=10, label="Line 1")
 
     gauss = DropShadowFilter(4)
 
-    for l in [l1, l2]:
+    for line in [line1, line2]:
 
         # draw shadows with same lines with slight offset.
-        xx = l.get_xdata()
-        yy = l.get_ydata()
+        xx = line.get_xdata()
+        yy = line.get_ydata()
         shadow, = ax.plot(xx, yy)
-        shadow.update_from(l)
+        shadow.update_from(line)
 
         # offset transform
-        ot = mtransforms.offset_copy(l.get_transform(), ax.figure,
-                                     x=4.0, y=-6.0, units='points')
-
-        shadow.set_transform(ot)
+        transform = mtransforms.offset_copy(
+            line.get_transform(), fig, x=4.0, y=-6.0, units='points')
+        shadow.set_transform(transform)
 
         # adjust zorder of the shadow lines so that it is drawn below the
         # original lines
-        shadow.set_zorder(l.get_zorder() - 0.5)
+        shadow.set_zorder(line.get_zorder() - 0.5)
         shadow.set_agg_filter(gauss)
         shadow.set_rasterized(True)  # to support mixed-mode renderers
 
@@ -194,7 +199,7 @@ def test_agg_filter():
 
 
 def test_too_large_image():
-    fig = plt.figure(figsize=(300, 1000))
+    fig = plt.figure(figsize=(300, 2**25))
     buff = io.BytesIO()
     with pytest.raises(ValueError):
         fig.savefig(buff)
@@ -217,7 +222,6 @@ def test_chunksize():
 
 @pytest.mark.backend('Agg')
 def test_jpeg_dpi():
-    Image = pytest.importorskip("PIL.Image")
     # Check that dpi is set correctly in jpg files.
     plt.plot([0, 1, 2], [0, 1, 0])
     buf = io.BytesIO()
@@ -227,7 +231,6 @@ def test_jpeg_dpi():
 
 
 def test_pil_kwargs_png():
-    Image = pytest.importorskip("PIL.Image")
     from PIL.PngImagePlugin import PngInfo
     buf = io.BytesIO()
     pnginfo = PngInfo()
@@ -238,11 +241,100 @@ def test_pil_kwargs_png():
 
 
 def test_pil_kwargs_tiff():
-    Image = pytest.importorskip("PIL.Image")
-    from PIL.TiffTags import TAGS_V2 as TAGS
     buf = io.BytesIO()
     pil_kwargs = {"description": "test image"}
     plt.figure().savefig(buf, format="tiff", pil_kwargs=pil_kwargs)
     im = Image.open(buf)
-    tags = {TAGS[k].name: v for k, v in im.tag_v2.items()}
+    tags = {TiffTags.TAGS_V2[k].name: v for k, v in im.tag_v2.items()}
     assert tags["ImageDescription"] == "test image"
+
+
+@pytest.mark.skipif(not features.check("webp"), reason="WebP support not available")
+def test_pil_kwargs_webp():
+    plt.plot([0, 1, 2], [0, 1, 0])
+    buf_small = io.BytesIO()
+    pil_kwargs_low = {"quality": 1}
+    plt.savefig(buf_small, format="webp", pil_kwargs=pil_kwargs_low)
+    assert len(pil_kwargs_low) == 1
+    buf_large = io.BytesIO()
+    pil_kwargs_high = {"quality": 100}
+    plt.savefig(buf_large, format="webp", pil_kwargs=pil_kwargs_high)
+    assert len(pil_kwargs_high) == 1
+    assert buf_large.getbuffer().nbytes > buf_small.getbuffer().nbytes
+
+
+@pytest.mark.skipif(not features.check("webp"), reason="WebP support not available")
+def test_webp_alpha():
+    plt.plot([0, 1, 2], [0, 1, 0])
+    buf = io.BytesIO()
+    plt.savefig(buf, format="webp", transparent=True)
+    im = Image.open(buf)
+    assert im.mode == "RGBA"
+
+
+def test_draw_path_collection_error_handling():
+    fig, ax = plt.subplots()
+    ax.scatter([1], [1]).set_paths(Path([(0, 1), (2, 3)]))
+    with pytest.raises(TypeError):
+        fig.canvas.draw()
+
+
+def test_chunksize_fails():
+    # NOTE: This test covers multiple independent test scenarios in a single
+    #       function, because each scenario uses ~2GB of memory and we don't
+    #       want parallel test executors to accidentally run multiple of these
+    #       at the same time.
+
+    N = 100_000
+    dpi = 500
+    w = 5*dpi
+    h = 6*dpi
+
+    # make a Path that spans the whole w-h rectangle
+    x = np.linspace(0, w, N)
+    y = np.ones(N) * h
+    y[::2] = 0
+    path = Path(np.vstack((x, y)).T)
+    # effectively disable path simplification (but leaving it "on")
+    path.simplify_threshold = 0
+
+    # setup the minimal GraphicsContext to draw a Path
+    ra = RendererAgg(w, h, dpi)
+    gc = ra.new_gc()
+    gc.set_linewidth(1)
+    gc.set_foreground('r')
+
+    gc.set_hatch('/')
+    with pytest.raises(OverflowError, match='cannot split hatched path'):
+        ra.draw_path(gc, path, IdentityTransform())
+    gc.set_hatch(None)
+
+    with pytest.raises(OverflowError, match='cannot split filled path'):
+        ra.draw_path(gc, path, IdentityTransform(), (1, 0, 0))
+
+    # Set to zero to disable, currently defaults to 0, but let's be sure.
+    with rc_context({'agg.path.chunksize': 0}):
+        with pytest.raises(OverflowError, match='Please set'):
+            ra.draw_path(gc, path, IdentityTransform())
+
+    # Set big enough that we do not try to chunk.
+    with rc_context({'agg.path.chunksize': 1_000_000}):
+        with pytest.raises(OverflowError, match='Please reduce'):
+            ra.draw_path(gc, path, IdentityTransform())
+
+    # Small enough we will try to chunk, but big enough we will fail to render.
+    with rc_context({'agg.path.chunksize': 90_000}):
+        with pytest.raises(OverflowError, match='Please reduce'):
+            ra.draw_path(gc, path, IdentityTransform())
+
+    path.should_simplify = False
+    with pytest.raises(OverflowError, match="should_simplify is False"):
+        ra.draw_path(gc, path, IdentityTransform())
+
+
+def test_non_tuple_rgbaface():
+    # This passes rgbaFace as a ndarray to draw_path.
+    fig = plt.figure()
+    fig.add_subplot(projection="3d").scatter(
+        [0, 1, 2], [0, 1, 2], path_effects=[patheffects.Stroke(linewidth=4)])
+    fig.canvas.draw()
